@@ -7,7 +7,7 @@ use parent qw/Exporter/;
 
 our @EXPORT = qw/retryX/;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 sub retryX {
     my (%args) = @_;
@@ -37,23 +37,27 @@ sub retryX {
         $lap->{each} = [gettimeofday];
         if (wantarray) {
             my @ret = eval { $code->($n) };
-            if ( _timeout($timeout, $lap) || !$retry_if->(@ret) ) {
+            unless ($retry_if->(@ret)) {
                 return @ret;
             }
+            _timeout($timeout, $lap);
         }
         elsif (not defined wantarray) {
             eval { $code->($n) };
-            if ( _timeout($timeout, $lap) || !$retry_if->() ) {
+            unless ($retry_if->()) {
                 return;
             }
+            _timeout($timeout, $lap);
         }
         else {
             my $ret = eval { $code->($n) };
-            if ( _timeout($timeout, $lap) || !$retry_if->($ret) ) {
+            unless ($retry_if->($ret)) {
                 return $ret;
             }
+            _timeout($timeout, $lap);
         }
         sleep $delay if $times; # Do not sleep in last time
+        _timeout($timeout, $lap);
     }
     die $err if $err;
 }
@@ -63,12 +67,12 @@ sub _timeout {
 
     if ( $timeout->{each}
             && tv_interval($lap->{each}) > $timeout->{each} ) {
-        return 1; # timeout!
+        die 'retry timeout: each time';
     }
 
     if ( $timeout->{total}
             && tv_interval($lap->{start}) > $timeout->{total} ) {
-        return 1; # timeout!
+        die 'retry timeout: total time';
     }
 
     return;
